//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_VEHICLE_H
#define PROTOCOL_VEHICLE_H

#include <thread>
#include "VehicleEndpoint.h"
#include "PlatoonService.h"
#include <NetworkingLib/Networking.h>
#include <NetworkingLib/ServiceServer.h>
#include <NetworkingLib/ServiceClient.h>
#include <NetworkingLib/Resolver.h>

namespace platoonProtocol
{
class Vehicle : public std::enable_shared_from_this<Vehicle>
{
public:
    using Ptr = std::shared_ptr<Vehicle>;

    using Callback = std::function<void()>;

    static constexpr std::uint16_t UDP_PORT = 10000;
    static constexpr std::uint16_t TCP_PORT = 10000;

    enum class Role
    {
        LEADER, FOLLOWER
    };

    virtual ~Vehicle()
    {}

    bool isIdle();

    bool isPlatoonRunning();

    void createPlatoon();

    void leavePlatoon();

    virtual void stop();

    void addVehicleEndpoint(VehicleId vehicleId, const std::string & host);

    VehicleEndpoint getOwnEndpoint() const
    { return ownEndpoint; }

    Role getRole() const noexcept
    { return role; }

    void setOnRunningPlatoonCallback(const Callback & callback)
    { onRunningPlatoonCallback = callback; }

    void setOnLeavingPlatoonCallback(const Callback & callback)
    { onLeavingPlatoonCallback = callback; }

protected:
    Vehicle(
        networking::Networking & net,
        VehicleId ownVehicleId,
        Role role);

    enum class State
    {
        IDLE,
        CREATING_PLATOON,
        RUNNING_PLATOON
    };

    static constexpr PlatoonConfig DEFAULT_PLATOON_CONFIG{0.0f, 0.0f};

    const VehicleEndpoint ownEndpoint;
    PlatoonId platoonId;
    std::atomic<PlatoonConfig> platoonConfig{DEFAULT_PLATOON_CONFIG};
    State state{State::IDLE};
    std::mutex stateMutex;
    networking::Networking & net;
    networking::service::Server<PlatoonService>::Ptr platoonCreateServiceServer;
    networking::service::Client<PlatoonService>::Ptr platoonCreateServiceClient;
    std::vector<VehicleEndpoint> vehicleEndpoints;
    std::vector<VehicleEndpoint>::iterator vehicleEndpointsIter;
    Callback onRunningPlatoonCallback;
    Callback onLeavingPlatoonCallback;

    virtual void doCreatePlatoon();

    virtual void doRunPlatoon() = 0;

    virtual void doLeavePlatoon() = 0;

    virtual void makePlatoonCreateRequest(const VehicleEndpoint & vehicleEndpoint) = 0;

    virtual void makePlatoonCreateResponse(const networking::Resolver::Endpoint & clientEndpoint,
                                           const PlatoonMessage & request,
                                           PlatoonMessage & response) = 0;

    void makePlatoonCreateRequestToNextEndpoint();

    void setState(State newState);

    VehicleEndpoint createVehicleEndpoint(VehicleId vehicleId, const std::string & host);

private:
    Role role;
};
}


#endif //PROTOCOL_VEHICLE_H
